

Application create: #PortManager with:
    (#( AbtViewApplication)
        collect: [:each | Smalltalk at: each ifAbsent: [
            Application errorPrerequisite: #PortManager missing: each]])!

PortManager becomeDefault!
Object subclass: #ParallelPort
    instanceVariableNames: 'port '
    classVariableNames: 'Default '
    poolDictionaries: ''!

PortManager becomeDefault!
Application subclass: #PortManager
    instanceVariableNames: ''
    classVariableNames: ''
    poolDictionaries: ''!


PortManager becomeDefault!

!ParallelPort class publicMethods !

clean

	Default := nil!

default
	"Answer the default, opened, instance of the receiver."

	Default isNil ifTrue: [self openDefault].
	^Default!

defaultPort
	" Return default parallel port address."

	^16r378!

fileName
	"Answer the host system file name for the library"

	^'io.dll'!

openDefault

	Default := self new! !

!ParallelPort publicMethods !

bitTransferSpeed
	" Current transfer speed for bits measured in Kbps.
	Warning: you should unplug your LPT connector."

	| time iterations bitsPerIteration |

	iterations := 10000.
	bitsPerIteration := 4.
	time := Time millisecondsToRun: [
			1 to: iterations do: [:i|
				self apiSetPortBit: self port location: 16r1.
				self apiGetPortBit: self port location: 16r1.
				self apiClrPortBit: self port location: 16r1.
				self apiGetPortBit: self port location: 16r1.]].

	^(((iterations * bitsPerIteration)/(time/1000))/1024) asFloat
!

byteTransferSpeed
	" Current transfer speed in byte chunks meuasured in Kbps.
	Warning: you should unplug your LPT connector."

	| time iterations bitsPerIteration |

	iterations := 10000.
	bitsPerIteration := 16.
	time := Time millisecondsToRun: [
			1 to: iterations do: [:i|
				self apiPortOut: self port data: 16rFF.
				self apiPortIn: self port]].

	^(((iterations * bitsPerIteration)/(time/1000))/1024) asFloat

!

clearBit: anInteger
	" Clear the bit indicated by anInteger (1-8) in the physical port of the receiver."

	^self apiClrPortBit: self port location: anInteger - 1!

clearPort
	" Set all port bits to zero."

	^self writePort: 16r00!

getBit: anInteger
	" Get the bit indicated by anInteger (1-8) in the physical port of the receiver."

	^((self apiGetPortBit: self port 
		 location: anInteger - 1) = -1) 
					ifTrue: [^true] 
					ifFalse: [^false]!

initializePort
	" Iniatialize the receiver with default port address."

	port := self class defaultPort.!

isDriverInstalled
	" Return true if the library is installed."

	^(self apiIsDriverInstalled ~= 0)!

port
	" Return the port the receiver is hooked to."

	port isNil ifTrue: [ self initializePort ].
	^port!

port: anInteger
	" Set the address of the physical port for the receiver (like 16r378)."

	port := anInteger!

readDWordPort
	" Return the current value read at the physical port of the receiver."

	^(self apiPortDWordIn: self port) asInteger!

readPort
	" Return the current value read at the physical port of the receiver."

	^(self apiPortIn: self port) asInteger!

readWordPort
	" Return the current value read at the physical port of the receiver."

	^(self apiPortWordIn: self port) asInteger!

setBit: anInteger
	" Set the bit indicated by anInteger (1-8) in the physical port of the receiver."

	^self apiSetPortBit: self port location: anInteger - 1!

setPort
	" Set all port bits to one."

	^self writePort: 16rFF
!

singleBitShift: sideBoolean inject: aBoolean
	" Shift the physical port bits by one. If sideBoolean is true
	shift to the left, if not to the right. If aBoolean is true inject ones
	on the opposite side, else inject zeros. Return MSB if shifting left, else return LSB."

	(sideBoolean)
		ifTrue: [ ^((self apiLeftPortShift: self port 
				       data: (aBoolean ifTrue: [1] ifFalse: [0])) = -1)]
		ifFalse: [ ^((self apiRightPortShift: self port 
				       data: (aBoolean ifTrue: [1] ifFalse: [0])) = -1)].!

toggleBit: anInteger
	" Toggle the bit indicated by anInteger (1-8) in the physical port of the receiver."

	^self apiNotPortBit: self port location: anInteger - 1!

writeDWordPort: anInteger
	" Write anInteger to the physical port of the receiver."

	^self apiPortDWordOut: self port data: anInteger!

writePort: aByte
	" Write aByte to the physical port of the receiver."

	^self apiPortOut: self port data: aByte!

writeWordPort: anInteger
	" Write anInteger to the physical port of the receiver."

	^self apiPortWordOut: self port data: anInteger! !

!ParallelPort privateMethods !

apiClrPortBit: aPort location: aByte
	" Clears the bit of the specified port (0-7)."

	<c: void 'io.dll':ClrPortBit int16 int8>
	^self primitiveFailed!

apiGetPortBit: aPort location: aByte
	" Returns the state of the specified bit.
	-1 means set, 0 means clear"

	<c: int16 'io.dll':GetPortBit int16 int8>
	^self primitiveFailed!

apiIsDriverInstalled
	" Returns non-zero if io.dll is installed and functioning. 
	The primary purpose of this function is to ensure that the kernel mode driver 
	for NT/2000/XP has been installed and is accessible."

	<c: int16 'io.dll':IsDriverInstalled>
	^self primitiveFailed!

apiLeftPortShift: aPort data: aWord
	" Shifts the specified port to the left. The MSB is returned, and the value passed becomes the LSB."

	<c: int16 'io.dll':LeftPortShift int16 int16>
	^self primitiveFailed!

apiNotPortBit: aPort location: aByte
	" Nots (inverts) the bit of the specified port."

	<c: void 'io.dll':NotPortBit int16 int8>
	^self primitiveFailed!

apiPortDWordIn: aPort
	" Reads a double word (32-bits) from the specified port." 

	<c: int32 'io.dll':PortDWordIn int16>
	^self primitiveFailed!

apiPortDWordOut: aPort data: anInteger
	" Outputs a double word (32-bits) to the specified port."

	<c: void 'io.dll':PortDWordOut int16 int32>
	^self primitiveFailed!

apiPortIn: aPort
	" Reads a byte from the specified port."

	<c: int8 'io.dll':PortIn int16>
	^self primitiveFailed!

apiPortOut: aPort data: aByte
	" Outputs a byte to the specified port."

	<c: void 'io.dll':PortOut int16 int8>
	^self primitiveFailed!

apiPortWordIn: aPort
	" Reads a word (16-bits) from the specified port."

	<c: int16 'io.dll':PortWordIn int16>
	^self primitiveFailed!

apiPortWordOut: aPort data: aWord
	" Outputs a word (16-bits) to the specified port."

	<c: void 'io.dll':PortWordOut int16 int16>
	^self primitiveFailed!

apiRightPortShift: aPort data: aWord
	" Shifts the specified port to the right. The LSB is returned, and the value passed becomes the MSB."

	<c: int16 'io.dll':RightPortShift int16 int16>
	^self primitiveFailed!

apiSetPortBit: aPort location: aByte
	" Sets the bit of the specified port (0-7)."

	<c: void 'io.dll':SetPortBit int16 int8>
	^self primitiveFailed! !

!PortManager class privateMethods !

abtIsViewApplication

	^true! !

ParallelPort initializeAfterLoad!
PortManager initializeAfterLoad!

PortManager loaded!